//
//  TelemetryPacketScanner.swift
//  snowm_scanner
//
//  Created by Brainants Work on 18/01/2021.
//

import Foundation
import CoreBluetooth
import SwiftyBluetooth

public class TelemetryPacketScanner: NSObject,CBCentralManagerDelegate{
    var canScan = false
    var cbCentralManager:CBCentralManager!
    var postLink:String = "https://api.genius.stratosfy.io/device/deviceData"
    static let Instance = TelemetryPacketScanner()
    var defaults:UserDefaults = UserDefaults.standard
    var oneHourInMills = 3600000

    
    private override init(){
        super.init()
        cbCentralManager = CBCentralManager.init(delegate: self, queue: nil)
    }
    
    
    private func saveSentDetails(packet: String) {
        defaults.set(Int(NSDate().timeIntervalSince1970), forKey: packet)
    }
    
    private func getSentDetails(packet: String) -> Int {
        var date = defaults.value(forKey: packet)
        if(date==nil){
            date=0
        }
        return date as! Int
    }
   
    
    func startScan(onPacket:@escaping(String)->(),syncWithServer:Bool) {
        SwiftyBluetooth.scanForPeripherals(withServiceUUIDs: nil,timeoutAfter: 99999) { [self] scanResult in
            switch scanResult {
            case .scanStarted:
                break
            case .scanResult( _, let advertisementData, _):
                let raw = "\(String(describing: advertisementData["kCBAdvDataManufacturerData"]))"
                if(raw != "nil")
                {
                    let finalRaw = self.getRawDataOnly(raw)
                    if(self.stratosfyCompanyCode(finalRaw)){
                        let callBackRawData = self.getRaw(raw: finalRaw)
                        onPacket(callBackRawData);
                        if(syncWithServer){
                            self.postReq(self.postLink, packet: callBackRawData)
                        }
                    }
                }
                
                break
                
            case .scanStopped( _,_):
                break
            }
        }
        
        
    }
    func cancelScan() {	
        if(canScan)
        {
            SwiftyBluetooth.stopScan()
        }
        
    }
    func getRaw(raw:String)->String{
        //currently all headers are same as per doc (also  of no use) and cannot get the header so this is currently hack for the headers
        return "0201061BFF"+raw.uppercased().removingWhitespaces();
    }
    
    public func centralManagerDidUpdateState(_ central: CBCentralManager) {
        switch central.state {
        case .unknown:
            print("central.state is .unknown")
        case .resetting:
            print("central.state is .resetting")
        case .unsupported:
            print("central.state is .unsupported")
        case .unauthorized:
            print("central.state is .unauthorized")
        case .poweredOff:
            print("central.state is .poweredOff")
        case .poweredOn:
            print("central.state is .poweredOn")
            canScan = true
        default:
            print("central.state missing")
        }
        
    }
    
    func getRawDataOnly(_ allData:String)->String{
        let midRaw = allData.components(separatedBy: "Optional(<")[1]
        return midRaw.components(separatedBy: ">)")[0]
    }
    
    func stratosfyCompanyCode(_ rawData:String)->Bool{
        let companyId = rawData.prefix(4)
        //change company code if required here
        return companyId == "8646"
    }
    
    
    func postReq(_ link:String,packet:String){
        if((Int(NSDate().timeIntervalSince1970)-self.getSentDetails(packet: packet))>self.oneHourInMills){
            
            
            let json = ["rawData":packet]
            let jsonData = try? JSONSerialization.data(withJSONObject: json)
            
            // create post request
            let url = URL(string: link)! //PUT Your URL
            var request = URLRequest(url: url)
            request.httpMethod = "POST"
            request.setValue("\(String(describing: jsonData?.count))", forHTTPHeaderField: "Content-Length")
            request.setValue("application/json", forHTTPHeaderField: "Content-Type")
            // insert json data to the request
            request.httpBody = jsonData
            
            let task = URLSession.shared.dataTask(with: request) { data, response, error in
                guard let data = data, error == nil else {
                    print(error?.localizedDescription ?? "No data")
                    return
                }
                let responseJSON = try? JSONSerialization.jsonObject(with: data, options: [])
                if let responseJSON = responseJSON as? [String: Any] {
                    self.saveSentDetails(packet: packet)
                    print(responseJSON) //Code after Successfull POST Request
                }
            }
            
            task.resume()
        }
    }
    
}

protocol RawPacketListener {
    func onPacket(rawData:String) -> Void
}



extension String {
    subscript(_ i: Int) -> String {
        let idx1 = index(startIndex, offsetBy: i)
        let idx2 = index(idx1, offsetBy: 1)
        return String(self[idx1..<idx2])
    }
}
extension Data {
    
    var hexString: String {
        return map { String(format: "%02hhx", $0) }.joined()
    }
    
    var decimalString: String {
        return map { String(format: "%d", $0) }.joined()
    }
    
}
